/*

 Copyright (C) 2006 NTT DATA Corporation

 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, version 2.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 */

package com.clustercontrol.performance.jmx;

import java.util.Collection;
import java.util.Iterator;

import javax.ejb.FinderException;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.performance.bean.CollectorRunStatus;
import com.clustercontrol.performance.ejb.bmp.CollectorLocal;
import com.clustercontrol.performance.ejb.bmp.CollectorLocalHome;
import com.clustercontrol.performance.util.JNDIConnectionManager;
import com.clustercontrol.util.apllog.AplLogger;

/**
 * 性能管理機能の実績収集、リソース監視機能用性能値収集を再実行するクラス
 *
 */
public class PerformanceRestartManager {
	protected static Log m_log = LogFactory.getLog( PerformanceRestartManager.class );
	
	/**
	 * 状態が「実行中」である収集を再開します
	 */
	public void restartAll() {
		try {
			CollectorLocalHome home = JNDIConnectionManager.getCollectorLocalHome();
			Collection<CollectorLocal> pks = home.findAll();

			Iterator<CollectorLocal> itr = pks.iterator();
			while (itr.hasNext()) {
				CollectorLocal collector = itr.next();

				if(collector.getStatus() == CollectorRunStatus.RUNNING){
					// 収集を再開します
					String collectorId = collector.getCollectorID();
					int collectorType = collector.getCollectorType();
					
					m_log.info("restart : collectorId = " + collectorId + ", collectorType = " + collectorType);
					boolean restarted = false;
					try{
						restarted = collector.restartCollect();
					} catch (Exception e) {
						m_log.warn("restartAll()", e);
					}
					
					if(restarted){
						m_log.info("restart : collectorId = " + collectorId + ", collectorType = " + collectorType + " success.");
						
					}else{
						m_log.warn("restart : collectorId = " + collectorId + ", collectorType = " + collectorType + " failed.");

						// 収集再開失敗をInternalに出力
						AplLogger apllog = new AplLogger("PERF", "perf");
						String[] args = {collectorId, Integer.toString(collectorType)};
						 apllog.put("SYS", "014", args);
					}
				}
			}
		} catch (FinderException e) {
			m_log.error("restartAll()", e);
		} catch (NamingException e) {
			m_log.error("restartAll()", e);
		}
	}
	
}
