package com.clustercontrol.snmptrap.ejb.mdb;

import java.util.ArrayList;
import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.bean.DataRangeConstant;
import com.clustercontrol.bean.HinemosModuleConstant;
import com.clustercontrol.bean.OutputBasicInfo;
import com.clustercontrol.bean.OutputNotifyGroupInfo;
import com.clustercontrol.bean.PriorityConstant;
import com.clustercontrol.commons.util.HinemosProperties;
import com.clustercontrol.commons.util.JmsPersistenceConfig;
import com.clustercontrol.commons.util.SendQueue;
import com.clustercontrol.notify.bean.QueueConstant;
import com.clustercontrol.repository.bean.FacilityTreeAttributeConstant;
import com.clustercontrol.repository.ejb.session.RepositoryControllerLocal;
import com.clustercontrol.repository.ejb.session.RepositoryControllerUtil;
import com.clustercontrol.snmptrap.bean.MonitorSnmpTrapInfo;
import com.clustercontrol.snmptrap.bean.MonitorSnmpTrapOidInfo;
import com.clustercontrol.snmptrap.bean.SnmpTrapMasterInfo;
import com.clustercontrol.snmptrap.ejb.entity.SnmpTrapMasterPK;
import com.clustercontrol.snmptrap.factory.SelectMonitorInfo;
import com.clustercontrol.snmptrap.factory.TrapSnmp;
import com.clustercontrol.snmptrap.factory.TrapSnmpManager;
import com.clustercontrol.snmptrap.message.SnmpTrapMessageInfo;
import com.clustercontrol.util.Messages;
import com.clustercontrol.util.apllog.AplLogger;
/**
 * <!-- begin-user-doc --> SNMPトラップを受信したときに処理を行う　Message Driven Bean '<em><b>LogOutputBean</b></em>'. <!-- end-user-doc --> *
 *
 * <!-- begin-xdoclet-definition -->
 * @ejb.bean name="NotifySnmpTrapBean" 
 *     acknowledge-mode="Auto-acknowledge"
 *     destination-type="javax.jms.Queue"
 *     
 *     transaction-type="Container"
 *     destination-jndi-name="queue/clustercontrolSnmpTrapSnmpTrapNotify"
 * 
 * @jboss.depends name="user:service=HinemosService,name=HinemosService"
 * 
 * @jboss.container-configuration
 *  name="Singleton Message Driven Bean"
 * 
 *--
 * Server Runtime Specific Tags
 * If you are not using a specific runtime, you can safely remove the tags below.
 * @jonas.message-driven-destination jndi-name="queue/clustercontrolSnmpTrapSnmpTrapNotify"
 * @jboss.destination-jndi-name name="queue/clustercontrolSnmpTrapSnmpTrapNotify"
 *
 *--
 * <!-- end-xdoclet-definition -->
 * @generated          
 */
public class NotifySnmpTrapBean implements MessageDrivenBean, MessageListener {
	
	protected static Log m_log = LogFactory.getLog( NotifySnmpTrapBean.class );
	
	/** 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * The context for the message-driven bean, set by the EJB container. 
	 * @generated
	 */
	private javax.ejb.MessageDrivenContext messageContext = null;
	
	/** 
	 * Required method for container to set context.
	 * @generated 
	 */
	public void setMessageDrivenContext(
			javax.ejb.MessageDrivenContext messageContext)
	throws javax.ejb.EJBException {
		this.messageContext = messageContext;
	}
	
	/** 
	 * Required creation method for message-driven beans. 
	 *
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 *
	 * <!-- begin-xdoclet-definition -->
	 * @ejb.create-method 
	 * <!-- end-xdoclet-definition -->
	 * @generated
	 */
	public void ejbCreate() {
		//no specific action required for message-driven beans 
	}
	
	/** 
	 * Required removal method for message-driven beans. 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void ejbRemove() {
		messageContext = null;
	}
	
	/** 
	 * トラップ情報を受信したときに処理するメソッド<Bean>
	 * 
	 *  これ以前にMBean上のメソッドで監視対象かチェックを
	 *  一部行う。その結果をJMSを通してこのメソッドに接続する。
	 */
	public void onMessage(Message message) {
		m_log.debug("onMessage() : start" );
		if(message instanceof ObjectMessage){
			ObjectMessage objectMessage = (ObjectMessage)message;
			
			Object obj = null;
			try {
				obj = objectMessage.getObject();
			} catch (JMSException e) {
				m_log.warn("onMessage() : The reception message is illegal.", e);
				return;
			}
			
			if(obj instanceof SnmpTrapMessageInfo){
				
				
				RepositoryControllerLocal repository = null;
				
				String trapOid = null;
				int genericId = 0;
				int specificId = 0;
				try{
					
					// レポジトリ
					repository = RepositoryControllerUtil.getLocalHome().create();
					
					// SNMPTRAP通知情報
					SnmpTrapMessageInfo info = (SnmpTrapMessageInfo)obj;
					String monitorId = info.getMonitorId();
					String facilityId = info.getFacilityId();
					String facilityPath ;
					
					//Unregisteredスコープの場合には、IPアドレスを入れる。
					if(info.getFacilityId().equals(FacilityTreeAttributeConstant.UNREGISTEREFD_SCOPE)){
						facilityPath = info.getAgentAdder();
					}else{
						facilityPath = repository.getFacilityPath(facilityId, null);
					}
					
					trapOid = info.getTrapOid();
					genericId = info.getGenericId();
					specificId = info.getSpecificId();
					m_log.debug("onMessage() : received trap :" + trapOid+":"+genericId+":"+specificId );
					SelectMonitorInfo select = new SelectMonitorInfo();
					MonitorSnmpTrapInfo snmpTrapInfo = select.getMonitorInfo(monitorId);
					
					//この後メッセージ送信時に使う変数軍
					//指定OIDではカスタマイズ可能であるため、この３つは個別に
					//変更する。
					String msg ;
					String msgOrg;
					int priority;
					
					if(snmpTrapInfo.getCheckMode() == MonitorSnmpTrapInfo.ALL_OID){
						m_log.debug("onMessage() : checkmode ALL_OID : search from masterdb. " );
						
						//マスター登録OIDすべての場合
						// trapOid, genericId,specificIdを元にマスターテーブルに問い合わせを行う。
						SnmpTrapMasterInfo master = select.findMasterInfo(trapOid,genericId,specificId);
						
						//問い合わせた結果が１つであれば、イベントを生成する処理を行う。		
						if(master != null && master.getMib() != null){
							//指定OID監視の場合には以下の３つの要素がカスタマイズ可能だが、
							//マスター登録OIDすべての場合にはその値もマスターから取得する。
							
							msg    = master.getLogmsg();
							msgOrg = master.getDescr();
							priority = master.getPriority();
							
							//メッセージ送信へ
							sendMassage( msg, msgOrg, info, master, priority,
									snmpTrapInfo,  facilityPath);
							m_log.debug("onMessage() : sendmessage ");
							
						}else{
							//一般トラップが1件もマッチしない場合は、再検索
							if(genericId != 6){//SNMPv1からSNMPv2形式の変換において、genericId=6以外(標準トラップ)の場合はOIDはTrapSnmp.GENERIC_TRAPSに従う
								master = select.findMasterInfo(TrapSnmp.GENERIC_TRAPS.get(genericId).toString(),genericId,specificId);
								if(master !=null && master.getMib() != null ){
									msg    = master.getLogmsg();
									msgOrg = master.getDescr();
									priority = master.getPriority();
								
									//メッセージ送信へ
									sendMassage( msg, msgOrg, info, master, priority,
											snmpTrapInfo,  facilityPath);
									m_log.debug("onMessage() : sendmessage( change to generic trap ) ");
								}
							}
							
						}
						
					}else if(snmpTrapInfo.getCheckMode() == MonitorSnmpTrapInfo.UNREGISTERED_OID){
						
						m_log.debug("onMessage() : unregisterd trap  " );
						
						//マスター未登録OIDの場合
						// trapOid, genericId,specificIdを元にマスターテーブルに問い合わせを行う。
						
						if ( genericId == TrapSnmp.ENTERPRISE_GENERIC_ID){
							
							SnmpTrapMasterInfo master = select.findMasterInfo(trapOid,genericId,specificId);
							
//							問い合わせた結果が0であれば、イベントを生成する処理を行う。						
							if(master != null && master.getMib() == null){
								
								//指定OID監視の場合には以下の３つの要素がカスタマイズ可能だが、
								//未登録MOIDの場合にはメッセージがない。重要度は常に不明
								
								msg    = "oid  : " + trapOid + " specificId : " + specificId +" generic_id : " + genericId;
								msgOrg =  "";
								priority = 1; // 重要度1は不明
								
								//メッセージ送信へ
								sendMassage( msg, msgOrg, info, master,  priority,
										snmpTrapInfo,  facilityPath);
								m_log.debug("onMessage() : sendmessage ");
							}
						}
						
						
					}else{
						//m_log.error("onMessage() : incoming messages : " + snmpTrapInfo.getCheckMode());
						m_log.debug("onMessage() : search from monitors. " );
						
						//指定OIDを監視する場合
						//
						ArrayList<MonitorSnmpTrapOidInfo> oidList = snmpTrapInfo.getOidInfos();
						
						int matched = 0;
						for (int i = 0; i < oidList.size(); i++) {
							MonitorSnmpTrapOidInfo oidInfo = oidList.get(i);
							
							// OID/GID/SIDが同一か
							if(trapOid.equals(oidInfo.getTrapOid())
									&&(genericId == oidInfo.getGenericId())
									&&(specificId == oidInfo.getSpecificId())){
								
								String mib = oidInfo.getMib();
								priority = oidInfo.getPriority();
								msg = oidInfo.getLogmsg();
								msgOrg = oidInfo.getDescr();
								
								// マスタ情報取得
								SnmpTrapMasterPK pk = new SnmpTrapMasterPK(										
										mib,
										trapOid,
										genericId,
										specificId);
								SnmpTrapMasterInfo master = select.getMasterInfo(pk);
								
								//メッセージ送信へ
								sendMassage( msg, msgOrg, info, master, priority,
										snmpTrapInfo,  facilityPath);
								matched++;
								m_log.debug("onMessage() : sendmessage ");
								break;
							}
						}
						// 一般トラップが１件もマッチしない場合に再検索
						if((matched==0)&&(genericId != TrapSnmp.ENTERPRISE_GENERIC_ID)){
							m_log.debug("onMessage() : re-search generic trap");
							for (int i = 0; i < oidList.size(); i++){
								MonitorSnmpTrapOidInfo oidInfo = oidList.get(i);
								
								if( oidInfo.getTrapOid().equals(TrapSnmp.GENERIC_TRAPS.get(genericId).toString())){
									
									String mib = oidInfo.getMib();
									priority = oidInfo.getPriority();
									msg = oidInfo.getLogmsg();
									msgOrg = oidInfo.getDescr();
									
									// マスタ情報取得
									SnmpTrapMasterPK pk = new SnmpTrapMasterPK(										
											mib,
											TrapSnmp.GENERIC_TRAPS.get(genericId).toString(),
											genericId,
											specificId);
									SnmpTrapMasterInfo master = select.getMasterInfo(pk);
									
									//メッセージ送信へ
									sendMassage( msg, msgOrg, info, master, priority,
											snmpTrapInfo,  facilityPath);
									
									m_log.debug("onMessage() : sendmessage ");
									
									break;
								}
								
							}
						}
					}
				}
				catch(EJBException e){
					m_log.error("onMessage(): The error occurred by the notification processing of SNMPTRAP. " +
							"trapOid = " + trapOid + ", specificId = " + specificId + ", genericId = " + genericId, e);
					
					// Internal Event
					AplLogger apllog = new AplLogger("TRAP", "trap");
					String[] args = {trapOid, String.valueOf(genericId), String.valueOf(specificId)};
					apllog.put("SYS", "009", args);
				}
				catch(Exception e){
					m_log.error("onMessage(): The error occurred by the notification processing of SNMPTRAP. " +
							"trapOid = " + trapOid + ", specificId = " + specificId + ", genericId = " + genericId, e);

					// Internal Event
					AplLogger apllog = new AplLogger("TRAP", "trap");
					String[] args = {trapOid, String.valueOf(genericId), String.valueOf(specificId)};
					apllog.put("SYS", "009", args);
				}
				finally{
					if(repository != null){
						try {
							repository.remove();
						} catch (Exception e) {
							m_log.warn("onMessage() : " +
									"The error occurred by remove of sessionbean. " , e);
						}
					}
					repository = null;
				}
			}
			else{
				m_log.warn("onMessage(): ObjectMessage is not an expected instance. " + obj.toString());
			}
		}
		m_log.debug("onMessage() : end ");
	}
	
	
	/**
	 * メッセージを詰めて送信するメソッド<BR>
	 * 
	 * * （長いのでOnMessageから分割）
	 * @param msg イベントを生成する時のメッセージ（）
	 * @param msgOrg イベントを生成する際のオリジナルメッセージ
	 * @param info 受信したトラップ情報
	 * @param master　受信情報に対応するマスターのSNMPトラップ情報
	 * @param monitorId マッチしたMonitorID
	 * @param priority イベントの設定される重要度
	 * @param snmpTrapInfo MonitorIdに対応する監視設定除法
	 * @param facilityId マッチしたファシリティID
	 * @param facilityPath ファシリティ名
	 * @param trapOid　受信したトラップのOID
	 */
	private void sendMassage(String msg, 
			String msgOrg, 
			SnmpTrapMessageInfo info,
			SnmpTrapMasterInfo master,
			int priority,
			MonitorSnmpTrapInfo snmpTrapInfo,
			
			String facilityPath
	){
		
		m_log.debug("sendMassage() start:" + info.getMonitorId() + " " + info.getFacilityId()  + " " + info.getTrapOid());
		// 送信Queueの初期化
		SendQueue sendQueue = null;
		
		String queueName = "";
		
		try {
			if (JmsPersistenceConfig.QUEUE_SNMPTRAP_MODE == DeliveryMode.NON_PERSISTENT) {
				queueName = QueueConstant.QUEUE_NAME_NOTIFYCONTROL_NON_PERSISTENCE;
			} else {
				queueName = QueueConstant.QUEUE_NAME_NOTIFYCONTROL;
			}
			if (m_log.isDebugEnabled()) {
				m_log.debug("initializing jms queue. (" + queueName + " "
						+ JmsPersistenceConfig.getPersistenceString(JmsPersistenceConfig.QUEUE_SNMPTRAP_MODE)
						+ " for " + HinemosModuleConstant.MONITOR_SNMPTRAP + ")");
			}
			sendQueue = new SendQueue(queueName);
		} catch (NamingException e1) {
			m_log.error("sendMassage()" +  e1.getMessage(), e1);
		} catch (JMSException e1) {
			m_log.error("sendMassage()" +  e1.getMessage(), e1);
		}
		
		
//		メッセージ "%parm[#n]%"をバインドされた値で置換
		if(msg != null && !"".equals(msg)){
			String[] values = info.getVarBindValue();
			for (int j = 0; j < values.length; j++) {
				String regex = TrapSnmpManager.REGEX_PREFIX + Integer.toString(j+1) + TrapSnmpManager.REGEX_SUFFIX;
				msg = msg.replace(regex, values[j]);
			}
		}
		else{
			msg = "";
		}
		
		// オリジナルメッセージ
		if(  msgOrg == null || "".equals(msgOrg)){
			// 空の場合、マスタ情報よりオリジナルメッセージを取得
			msgOrg = master.getDescr();
		}
		
		// オリジナルメッセージ "%parm[#n]%"をバインドされた値で置換
		if(msgOrg != null && !"".equals(msgOrg)){
			String[] values = info.getVarBindValue();
			for (int j = 0; j < values.length; j++) {
				String regex = TrapSnmpManager.REGEX_PREFIX + Integer.toString(j+1) + TrapSnmpManager.REGEX_SUFFIX;
				msgOrg = msgOrg.replace(regex, values[j]);
			}
		}
		else{
			msgOrg = "";
		}
		
		// 監視管理への通知メッセージ作成
		OutputNotifyGroupInfo notifyInfo = new OutputNotifyGroupInfo();
		
		notifyInfo.setMonitorId(info.getMonitorId());
		notifyInfo.setPluginId(HinemosModuleConstant.MONITOR_SNMPTRAP);
		notifyInfo.setPriority(priority);
		
		notifyInfo.setNotifyGroupId(snmpTrapInfo.getNotifyGroupId());
		notifyInfo.setApplication(snmpTrapInfo.getApplication());
		notifyInfo.setFacilityId(info.getFacilityId());
		notifyInfo.setScopeText(facilityPath);
		
		notifyInfo.setGenerationDate(info.getTrapDate());
		
		String messageId = null;
		if(priority == PriorityConstant.TYPE_INFO){
			messageId = TrapSnmpManager.MESSAGE_ID_INFO;
		}
		else if(priority == PriorityConstant.TYPE_WARNING){
			messageId = TrapSnmpManager.MESSAGE_ID_WARNING;
		}
		else if(priority == PriorityConstant.TYPE_CRITICAL){
			messageId = TrapSnmpManager.MESSAGE_ID_CRITICAL;
		}
		else{
			messageId = TrapSnmpManager.MESSAGE_ID_UNKNOWN;
		}
		notifyInfo.setMessageId(messageId);
		
		// msgのうち最初の255文字までを通知する。
		m_log.debug("sendMassage() : message length "+Integer.toString(msg.length()) );
		if (msg.length()>DataRangeConstant.VARCHAR_256-1 ){
			notifyInfo.setMessage(msg.substring(0,DataRangeConstant.VARCHAR_256-1));
		}else{
			notifyInfo.setMessage(msg);
		}
		
		String[] args = {info.getTrapOid(), master.getUei()};
		notifyInfo.setMessageOrg(Messages.getString("message.snmptrap.3", args) + "\n" + msgOrg);
		
		// NULL文字などを置換する。
		try {
			convertMessage(notifyInfo);
		} catch (Exception e) {
			m_log.error("sendMessage : convertMessage failed " + e.getMessage());
		}
		
		try {
			//Queueに送信
			if (m_log.isDebugEnabled()) {
				m_log.debug("sending message to jms. (" + notifyInfo.getPluginId()
						+ " " + JmsPersistenceConfig.getPersistenceString(JmsPersistenceConfig.QUEUE_SNMPTRAP_MODE)
						+ " : notifyGroupId=" + notifyInfo.getNotifyGroupId() + " priority=" + notifyInfo.getPriority()
						+ " generationDate=" + notifyInfo.getGenerationDate() + " pluginId=" + notifyInfo.getPluginId()
						+ " monitorId=" + notifyInfo.getMonitorId() + " facilityId=" + notifyInfo.getFacilityId() + ")");
			}
			sendQueue.put(notifyInfo, JmsPersistenceConfig.QUEUE_SNMPTRAP_MODE);
		} catch (Exception e) {
			m_log.error("onMessage() : OutputNotifyGroupInfo send error : " + e.getMessage(), e);
			
		} finally {
			if (sendQueue != null) {
				try {
					sendQueue.terminate();
				} catch (JMSException e) {
					m_log.error("Terminate SendQueue Failed : " + e.getMessage(), e);
				}
			}
		}
	}
	
	// NULL文字等を置換するメソッド。
	private static void convertMessage(OutputBasicInfo info) {
		String messageStr = info.getMessage();
		String messageOrg = info.getMessageOrg();
		String oldCharList = HinemosProperties.getProperty("common.notify.replace.before", "0");
		char newChar = HinemosProperties.getProperty("common.notify.replace.after", "?").charAt(0);
		boolean convertMsgFlag = false;
		boolean convertMsgOrgFlag = false;
		for (String oldCharStr : oldCharList.split(",")) {
			int oldCharInt = Integer.parseInt(oldCharStr);
			Character oldChar = (char)oldCharInt;
			if (messageStr != null) {
				if (messageStr.contains(oldChar.toString())) {
					convertMsgFlag = true;
					messageStr = messageStr.replace(oldChar, newChar);
				}
			}
			if (messageOrg != null) {
				if (messageOrg.contains(oldChar.toString())) {
					convertMsgOrgFlag = true;
					messageOrg = messageOrg.replace(oldChar, newChar);
				}
			}
		}
		if (convertMsgFlag) {
			info.setMessage(messageStr);
		}
		if (convertMsgOrgFlag) {
			info.setMessageOrg(messageOrg);
		}
		if (convertMsgFlag || convertMsgOrgFlag) {
			m_log.info("convertMessage : convert is successful. " +
				convertMsgFlag + ", " + convertMsgOrgFlag);
		}
	}
}
